@@ -8,8 +8,8 @@ import (
 	"os"
 	"os/signal"
 	"path/filepath"
-	"runtime"
 	"strings"
+	"sync"
 	"syscall"
 	"text/tabwriter"
 	"text/template"
@@ -17,6 +17,7 @@ import (
 
 	"k8s.io/klog"
 
+	goovn "github.com/ebay/go-ovn"
 	"github.com/urfave/cli/v2"
 	"gopkg.in/fsnotify/fsnotify.v1"
 
@@ -147,7 +148,7 @@ func setupPIDFile(pidfile string) error {
 	// Create if it doesn't exist, else exit with error
 	if os.IsNotExist(err) {
 		if err := ioutil.WriteFile(pidfile, []byte(fmt.Sprintf("%d", os.Getpid())), 0644); err != nil {
-			klog.Errorf("failed to write pidfile %s (%v). Ignoring..", pidfile, err)
+			klog.Errorf("Failed to write pidfile %s (%v). Ignoring..", pidfile, err)
 		}
 	} else {
 		// get the pid and see if it exists
@@ -159,7 +160,7 @@ func setupPIDFile(pidfile string) error {
 		if os.IsNotExist(err1) {
 			// Left over pid from dead process
 			if err := ioutil.WriteFile(pidfile, []byte(fmt.Sprintf("%d", os.Getpid())), 0644); err != nil {
-				klog.Errorf("failed to write pidfile %s (%v). Ignoring..", pidfile, err)
+				klog.Errorf("Failed to write pidfile %s (%v). Ignoring..", pidfile, err)
 			}
 		} else {
 			return fmt.Errorf("pidfile %s exists and ovnkube is running", pidfile)
@@ -188,13 +189,13 @@ func runOvnKube(ctx *cli.Context) error {
 		return fmt.Errorf("failed to initialize exec helper: %v", err)
 	}
 
-	clientset, err := util.NewClientset(&config.Kubernetes)
+	clientset, egressIPClientset, egressFirewallClientset, crdClientset, err := util.NewClientsets(&config.Kubernetes)
 	if err != nil {
 		return err
 	}
 
 	// create factory and start the controllers asked for
-	factory, err := factory.NewWatchFactory(clientset)
+	factory, err := factory.NewWatchFactory(clientset, egressIPClientset, egressFirewallClientset, crdClientset)
 	if err != nil {
 		return err
 	}
@@ -225,15 +226,27 @@ func runOvnKube(ctx *cli.Context) error {
 	}
 
 	stopChan := make(chan struct{})
+	wg := &sync.WaitGroup{}
 
 	if master != "" {
-		if runtime.GOOS == "windows" {
-			return fmt.Errorf("Windows is not supported as a master")
+		var ovnNBClient, ovnSBClient goovn.Client
+		var err error
+
+		if ovnNBClient, err = util.NewOVNNBClient(); err != nil {
+			return fmt.Errorf("error when trying to initialize go-ovn NB client: %v", err)
 		}
+
+		if ovnSBClient, err = util.NewOVNSBClient(); err != nil {
+			return fmt.Errorf("error when trying to initialize go-ovn SB client: %v", err)
+		}
+
 		// register prometheus metrics exported by the master
-		metrics.RegisterMasterMetrics()
-		ovnController := ovn.NewOvnController(clientset, factory, stopChan)
-		if err := ovnController.Start(clientset, master); err != nil {
+		// this must be done prior to calling controller start
+		// since we capture some metrics in Start()
+		metrics.RegisterMasterMetrics(ovnNBClient, ovnSBClient)
+
+		ovnController := ovn.NewOvnController(clientset, egressIPClientset, egressFirewallClientset, factory, stopChan, nil, ovnNBClient, ovnSBClient, util.EventRecorder(clientset))
+		if err := ovnController.Start(clientset, master, wg); err != nil {
 			return err
 		}
 	}
@@ -244,11 +257,9 @@ func runOvnKube(ctx *cli.Context) error {
 		}
 		// register ovnkube node specific prometheus metrics exported by the node
 		metrics.RegisterNodeMetrics()
-		// register ovn specific (ovn-controller and ovn-northd) metrics
-		metrics.RegisterOvnMetrics()
 		start := time.Now()
-		n := ovnnode.NewNode(clientset, factory, node, stopChan)
-		if err := n.Start(); err != nil {
+		n := ovnnode.NewNode(clientset, factory, node, stopChan, util.EventRecorder(clientset))
+		if err := n.Start(wg); err != nil {
 			return err
 		}
 		end := time.Since(start)
@@ -256,15 +267,22 @@ func runOvnKube(ctx *cli.Context) error {
 	}
 
 	// now that ovnkube master/node are running, lets expose the metrics HTTP endpoint if configured
-	// start the prometheus server
+	// start the prometheus server to serve OVN K8s Metrics (default master port: 9409, node port: 9410)
 	if config.Kubernetes.MetricsBindAddress != "" {
 		metrics.StartMetricsServer(config.Kubernetes.MetricsBindAddress, config.Kubernetes.MetricsEnablePprof)
 	}
 
+	// start the prometheus server to serve OVN Metrics (default port: 9476)
+	if config.Kubernetes.OVNMetricsBindAddress != "" {
+		metrics.RegisterOvnMetrics(clientset, node)
+		metrics.StartOVNMetricsServer(config.Kubernetes.OVNMetricsBindAddress)
+	}
+
 	// run until cancelled
 	<-ctx.Context.Done()
 	close(stopChan)
 	factory.Shutdown()
+	wg.Wait()
 	return nil
 }
 
@@ -300,7 +318,7 @@ func watchForChanges(configPath string) error {
 				if !ok {
 					return
 				}
-				klog.Errorf("fsnotify error %v", err)
+				klog.Errorf("Error watching for changes to configmap: %s, err: %v", configPath, err)
 			}
 		}
 	}()
