@@ -5,11 +5,11 @@ package cni
 import (
 	"fmt"
 	"io/ioutil"
+	"net"
 	"os"
 	"os/exec"
 	"strconv"
 	"strings"
-	"time"
 
 	"k8s.io/klog"
 
@@ -20,23 +20,65 @@ import (
 	"github.com/containernetworking/plugins/pkg/ip"
 	"github.com/containernetworking/plugins/pkg/ns"
 	"github.com/vishvananda/netlink"
-
-	"k8s.io/apimachinery/pkg/util/wait"
 )
 
+type CNIPluginLibOps interface {
+	AddRoute(ipn *net.IPNet, gw net.IP, dev netlink.Link) error
+	SetupVeth(contVethName string, mtu int, hostNS ns.NetNS) (net.Interface, net.Interface, error)
+}
+
+type defaultCNIPluginLibOps struct{}
+
+var cniPluginLibOps CNIPluginLibOps = &defaultCNIPluginLibOps{}
+
+func (defaultCNIPluginLibOps) AddRoute(ipn *net.IPNet, gw net.IP, dev netlink.Link) error {
+	return ip.AddRoute(ipn, gw, dev)
+}
+
+func (defaultCNIPluginLibOps) SetupVeth(contVethName string, mtu int, hostNS ns.NetNS) (net.Interface, net.Interface, error) {
+	return ip.SetupVeth(contVethName, mtu, hostNS)
+}
+
+type SriovNetLibOps interface {
+	GetNetDevicesFromPci(pciAddress string) ([]string, error)
+	GetUplinkRepresentor(vfPciAddress string) (string, error)
+	GetVfIndexByPciAddress(vfPciAddress string) (int, error)
+	GetVfRepresentor(uplink string, vfIndex int) (string, error)
+}
+
+type defaultSRIOVLibOps struct{}
+
+var sriovLibOps SriovNetLibOps = &defaultSRIOVLibOps{}
+
+func (defaultSRIOVLibOps) GetNetDevicesFromPci(pciAddress string) ([]string, error) {
+	return sriovnet.GetNetDevicesFromPci(pciAddress)
+}
+
+func (defaultSRIOVLibOps) GetUplinkRepresentor(vfPciAddress string) (string, error) {
+	return sriovnet.GetUplinkRepresentor(vfPciAddress)
+}
+
+func (defaultSRIOVLibOps) GetVfIndexByPciAddress(vfPciAddress string) (int, error) {
+	return sriovnet.GetVfIndexByPciAddress(vfPciAddress)
+}
+
+func (defaultSRIOVLibOps) GetVfRepresentor(uplink string, vfIndex int) (string, error) {
+	return sriovnet.GetVfRepresentor(uplink, vfIndex)
+}
+
 func renameLink(curName, newName string) error {
-	link, err := netlink.LinkByName(curName)
+	link, err := util.GetNetLinkOps().LinkByName(curName)
 	if err != nil {
 		return err
 	}
 
-	if err := netlink.LinkSetDown(link); err != nil {
+	if err := util.GetNetLinkOps().LinkSetDown(link); err != nil {
 		return err
 	}
-	if err := netlink.LinkSetName(link, newName); err != nil {
+	if err := util.GetNetLinkOps().LinkSetName(link, newName); err != nil {
 		return err
 	}
-	if err := netlink.LinkSetUp(link); err != nil {
+	if err := util.GetNetLinkOps().LinkSetUp(link); err != nil {
 		return err
 	}
 
@@ -48,13 +90,13 @@ func setSysctl(sysctl string, newVal int) error {
 }
 
 func moveIfToNetns(ifname string, netns ns.NetNS) error {
-	vfDev, err := netlink.LinkByName(ifname)
+	vfDev, err := util.GetNetLinkOps().LinkByName(ifname)
 	if err != nil {
 		return fmt.Errorf("failed to lookup vf device %v: %q", ifname, err)
 	}
 
 	// move VF device to ns
-	if err = netlink.LinkSetNsFd(vfDev, int(netns.Fd())); err != nil {
+	if err = util.GetNetLinkOps().LinkSetNsFd(vfDev, int(netns.Fd())); err != nil {
 		return fmt.Errorf("failed to move device %+v to netns: %q", ifname, err)
 	}
 
@@ -62,22 +104,32 @@ func moveIfToNetns(ifname string, netns ns.NetNS) error {
 }
 
 func setupNetwork(link netlink.Link, ifInfo *PodInterfaceInfo) error {
-	if err := netlink.LinkSetHardwareAddr(link, ifInfo.MAC); err != nil {
+	// set the mac addresss, set down the interface before changing the mac
+	// so the EUI64 link local address generated uses the new MAC when we set it up again
+	if err := util.GetNetLinkOps().LinkSetDown(link); err != nil {
+		return fmt.Errorf("failed to set down interface %s: %v", link.Attrs().Name, err)
+	}
+	if err := util.GetNetLinkOps().LinkSetHardwareAddr(link, ifInfo.MAC); err != nil {
 		return fmt.Errorf("failed to add mac address %s to %s: %v", ifInfo.MAC, link.Attrs().Name, err)
 	}
+	if err := util.GetNetLinkOps().LinkSetUp(link); err != nil {
+		return fmt.Errorf("failed to set up interface %s: %v", link.Attrs().Name, err)
+	}
+
+	// set the IP address
 	for _, ip := range ifInfo.IPs {
 		addr := &netlink.Addr{IPNet: ip}
-		if err := netlink.AddrAdd(link, addr); err != nil {
+		if err := util.GetNetLinkOps().AddrAdd(link, addr); err != nil {
 			return fmt.Errorf("failed to add IP addr %s to %s: %v", ip, link.Attrs().Name, err)
 		}
 	}
 	for _, gw := range ifInfo.Gateways {
-		if err := ip.AddRoute(nil, gw, link); err != nil {
+		if err := cniPluginLibOps.AddRoute(nil, gw, link); err != nil {
 			return fmt.Errorf("failed to add gateway route: %v", err)
 		}
 	}
 	for _, route := range ifInfo.Routes {
-		if err := ip.AddRoute(route.Dest, route.NextHop, link); err != nil {
+		if err := cniPluginLibOps.AddRoute(route.Dest, route.NextHop, link); err != nil {
 			return fmt.Errorf("failed to add pod route %v via %v: %v", route.Dest, route.NextHop, err)
 		}
 	}
@@ -92,14 +144,14 @@ func setupInterface(netns ns.NetNS, containerID, ifName string, ifInfo *PodInter
 	var oldHostVethName string
 	err := netns.Do(func(hostNS ns.NetNS) error {
 		// create the veth pair in the container and move host end into host netns
-		hostVeth, containerVeth, err := ip.SetupVeth(ifName, ifInfo.MTU, hostNS)
+		hostVeth, containerVeth, err := cniPluginLibOps.SetupVeth(ifName, ifInfo.MTU, hostNS)
 		if err != nil {
 			return err
 		}
 		hostIface.Mac = hostVeth.HardwareAddr.String()
 		contIface.Name = containerVeth.Name
 
-		link, err := netlink.LinkByName(contIface.Name)
+		link, err := util.GetNetLinkOps().LinkByName(contIface.Name)
 		if err != nil {
 			return fmt.Errorf("failed to lookup %s: %v", contIface.Name, err)
 		}
@@ -134,7 +186,7 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName string, ifInfo *Pod
 	contIface := &current.Interface{}
 
 	// 1. get VF netdevice from PCI
-	vfNetdevices, err := sriovnet.GetNetDevicesFromPci(pciAddrs)
+	vfNetdevices, err := sriovLibOps.GetNetDevicesFromPci(pciAddrs)
 	if err != nil {
 		return nil, nil, err
 
@@ -147,19 +199,19 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName string, ifInfo *Pod
 	vfNetdevice := vfNetdevices[0]
 
 	// 2. get Uplink netdevice
-	uplink, err := sriovnet.GetUplinkRepresentor(pciAddrs)
+	uplink, err := sriovLibOps.GetUplinkRepresentor(pciAddrs)
 	if err != nil {
 		return nil, nil, err
 	}
 
 	// 3. get VF index from PCI
-	vfIndex, err := sriovnet.GetVfIndexByPciAddress(pciAddrs)
+	vfIndex, err := sriovLibOps.GetVfIndexByPciAddress(pciAddrs)
 	if err != nil {
 		return nil, nil, err
 	}
 
 	// 4. lookup representor
-	rep, err := sriovnet.GetVfRepresentor(uplink, vfIndex)
+	rep, err := sriovLibOps.GetVfRepresentor(uplink, vfIndex)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -170,14 +222,14 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName string, ifInfo *Pod
 	if err = renameLink(oldHostRepName, hostIface.Name); err != nil {
 		return nil, nil, fmt.Errorf("failed to rename %s to %s: %v", oldHostRepName, hostIface.Name, err)
 	}
-	link, err := netlink.LinkByName(hostIface.Name)
+	link, err := util.GetNetLinkOps().LinkByName(hostIface.Name)
 	if err != nil {
 		return nil, nil, err
 	}
 	hostIface.Mac = link.Attrs().HardwareAddr.String()
 
 	// 6. set MTU on VF representor
-	if err = netlink.LinkSetMTU(link, ifInfo.MTU); err != nil {
+	if err = util.GetNetLinkOps().LinkSetMTU(link, ifInfo.MTU); err != nil {
 		return nil, nil, fmt.Errorf("failed to set MTU on %s: %v", hostIface.Name, err)
 	}
 
@@ -193,15 +245,15 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName string, ifInfo *Pod
 		if err != nil {
 			return err
 		}
-		link, err = netlink.LinkByName(contIface.Name)
+		link, err = util.GetNetLinkOps().LinkByName(contIface.Name)
 		if err != nil {
 			return err
 		}
-		err = netlink.LinkSetMTU(link, ifInfo.MTU)
+		err = util.GetNetLinkOps().LinkSetMTU(link, ifInfo.MTU)
 		if err != nil {
 			return err
 		}
-		err = netlink.LinkSetUp(link)
+		err = util.GetNetLinkOps().LinkSetUp(link)
 		if err != nil {
 			return err
 		}
@@ -223,20 +275,6 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName string, ifInfo *Pod
 	return hostIface, contIface, nil
 }
 
-var iptablesCommands = [][]string{
-	// Block MCS
-	{"-A", "OUTPUT", "-p", "tcp", "-m", "tcp", "--dport", "22623", "--syn", "-j", "REJECT"},
-	{"-A", "OUTPUT", "-p", "tcp", "-m", "tcp", "--dport", "22624", "--syn", "-j", "REJECT"},
-	{"-A", "FORWARD", "-p", "tcp", "-m", "tcp", "--dport", "22623", "--syn", "-j", "REJECT"},
-	{"-A", "FORWARD", "-p", "tcp", "-m", "tcp", "--dport", "22624", "--syn", "-j", "REJECT"},
-
-	// Block cloud provider metadata IP except DNS
-	{"-A", "OUTPUT", "-p", "tcp", "-m", "tcp", "-d", "169.254.169.254", "!", "--dport", "53", "-j", "REJECT"},
-	{"-A", "OUTPUT", "-p", "udp", "-m", "udp", "-d", "169.254.169.254", "!", "--dport", "53", "-j", "REJECT"},
-	{"-A", "FORWARD", "-p", "tcp", "-m", "tcp", "-d", "169.254.169.254", "!", "--dport", "53", "-j", "REJECT"},
-	{"-A", "FORWARD", "-p", "udp", "-m", "udp", "-d", "169.254.169.254", "!", "--dport", "53", "-j", "REJECT"},
-}
-
 // ConfigureInterface sets up the container interface
 func (pr *PodRequest) ConfigureInterface(namespace string, podName string, ifInfo *PodInterfaceInfo) ([]*current.Interface, error) {
 	netns, err := ns.GetNS(pr.Netns)
@@ -268,7 +306,7 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, ifInf
 	uuids, _ := ovsFind("Interface", "_uuid", "external-ids:iface-id="+ifaceID)
 	for _, uuid := range uuids {
 		if out, err := ovsExec("remove", "Interface", uuid, "external-ids", "iface-id"); err != nil {
-			klog.Warningf("failed to clear stale OVS port %q iface-id %q: %v\n  %q", uuid, ifaceID, err, out)
+			klog.Warningf("Failed to clear stale OVS port %q iface-id %q: %v\n  %q", uuid, ifaceID, err, out)
 		}
 	}
 
@@ -296,8 +334,7 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, ifInf
 	}
 
 	if ifInfo.Ingress > 0 || ifInfo.Egress > 0 {
-		var l netlink.Link
-		l, err = netlink.LinkByName(hostIface.Name)
+		l, err := netlink.LinkByName(hostIface.Name)
 		if err != nil {
 			return nil, fmt.Errorf("failed to find host veth interface %s: %v", hostIface.Name, err)
 		}
@@ -311,29 +348,39 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, ifInf
 		}
 	}
 
+	// OCP HACK: block access to MCS/metadata; https://github.com/openshift/ovn-kubernetes/pull/19
+	err = setupIPTablesBlocks(netns, ifInfo)
+	if err != nil {
+		return nil, err
+	}
+	// END OCP HACK
+
 	err = netns.Do(func(hostNS ns.NetNS) error {
-		// Block access to certain things
-		for _, args := range iptablesCommands {
-			out, err := exec.Command("iptables", args...).CombinedOutput()
+		// deny IPv6 neighbor solicitations
+		dadSysctlIface := fmt.Sprintf("/proc/sys/net/ipv6/conf/%s/dad_transmits", contIface.Name)
+		if _, err := os.Stat(dadSysctlIface); !os.IsNotExist(err) {
+			err = setSysctl(dadSysctlIface, 0)
 			if err != nil {
-				return fmt.Errorf("could not set up pod iptables rules: %s", string(out))
+				klog.Warningf("Failed to disable IPv6 DAD: %q", err)
+			}
+		}
+		// generate address based on EUI64
+		genSysctlIface := fmt.Sprintf("/proc/sys/net/ipv6/conf/%s/addr_gen_mode", contIface.Name)
+		if _, err := os.Stat(genSysctlIface); !os.IsNotExist(err) {
+			err = setSysctl(genSysctlIface, 0)
+			if err != nil {
+				klog.Warningf("Failed to set IPv6 address generation mode to EUI64: %q", err)
 			}
 		}
 
-		if _, err := os.Stat("/proc/sys/net/ipv6/conf/all/dad_transmits"); !os.IsNotExist(err) {
-			err = setSysctl("/proc/sys/net/ipv6/conf/all/dad_transmits", 0)
-			if err != nil {
-				klog.Warningf("failed to disable IPv6 DAD: %q", err)
-			}
-		}
 		return ip.SettleAddresses(contIface.Name, 10)
 	})
 	if err != nil {
-		klog.Warningf("failed to settle addresses: %q", err)
+		klog.Warningf("Failed to settle addresses: %q", err)
 	}
 
 	if err = waitForPodFlows(ifInfo.MAC.String()); err != nil {
-		return nil, fmt.Errorf("timed out dumping br-int flow entries for sandbox: %v", err)
+		return nil, fmt.Errorf("timed out waiting for pod flows for pod: %s, error: %v", podName, err)
 	}
 
 	return []*current.Interface{hostIface, contIface}, nil
@@ -345,7 +392,7 @@ func (pr *PodRequest) deletePodConntrack() {
 	}
 	result, err := current.NewResultFromResult(pr.CNIConf.PrevResult)
 	if err != nil {
-		klog.Warningf("could not convert result to current version: %v", err)
+		klog.Warningf("Could not convert result to current version: %v", err)
 		return
 	}
 
@@ -366,26 +413,6 @@ func (pr *PodRequest) deletePodConntrack() {
 	}
 }
 
-// OCP HACK: wait for OVN to fully process the new pod
-func waitForPodFlows(mac string) error {
-
-	return wait.PollImmediate(200*time.Millisecond, 20*time.Second, func() (bool, error) {
-		// Query the flows by mac address
-		query := fmt.Sprintf("table=9,dl_src=%s",mac)
-		// ovs-ofctl dumps error on stderr, so stdout will only dump flow data if matches the query.
-		stdout, err := ofctlExec("dump-flows", "br-int", query)
-		if err != nil {
-			return false, nil
-		}
-		if len(stdout) == 0 {
-			return false, nil
-		}
-		return true, nil
-	})
-}
-
-// END OCP HACK
-
 // PlatformSpecificCleanup deletes the OVS port
 func (pr *PodRequest) PlatformSpecificCleanup() error {
 	ifaceName := pr.SandboxID[:15]
@@ -395,7 +422,7 @@ func (pr *PodRequest) PlatformSpecificCleanup() error {
 	out, err := exec.Command("ovs-vsctl", ovsArgs...).CombinedOutput()
 	if err != nil && !strings.Contains(string(out), "no port named") {
 		// DEL should be idempotent; don't return an error just log it
-		klog.Warningf("failed to delete OVS port %s: %v\n  %q", ifaceName, err, string(out))
+		klog.Warningf("Failed to delete OVS port %s: %v\n  %q", ifaceName, err, string(out))
 	}
 
 	_ = clearPodBandwidth(pr.SandboxID)
