@@ -41,7 +41,8 @@ import (
 )
 
 const (
-	egressfirewallCRD = "egressfirewalls.k8s.ovn.org"
+	egressfirewallCRD    string = "egressfirewalls.k8s.ovn.org"
+	clusterPortGroupName string = "clusterPortGroup"
 )
 
 // ServiceVIPKey is used for looking up service namespace information for a
@@ -104,11 +105,14 @@ type namespaceInfo struct {
 
 // eNode is a cache helper used for egress IP assignment
 type eNode struct {
-	v4Subnet    *net.IPNet
-	v6Subnet    *net.IPNet
-	allocations map[string]bool
-	tainted     bool
-	name        string
+	v4IP               net.IP
+	v6IP               net.IP
+	v4Subnet           *net.IPNet
+	v6Subnet           *net.IPNet
+	allocations        map[string]bool
+	isEgressAssignable bool
+	tainted            bool
+	name               string
 }
 
 // Controller structure is the object which holds the controls for starting
@@ -152,6 +156,9 @@ type Controller struct {
 	// An address set factory that creates address sets
 	addressSetFactory AddressSetFactory
 
+	// Port group for all cluster logical switch ports
+	clusterPortGroupUUID string
+
 	// Port group for ingress deny rule
 	portGroupIngressDeny string
 
@@ -784,6 +791,9 @@ func (oc *Controller) WatchEgressNodes() {
 		UpdateFunc: func(old, new interface{}) {
 			oldNode := old.(*kapi.Node)
 			newNode := new.(*kapi.Node)
+			if err := oc.initEgressIPAllocator(newNode); err != nil {
+				klog.Error(err)
+			}
 			oldLabels := oldNode.GetLabels()
 			newLabels := newNode.GetLabels()
 			_, oldHadEgressLabel := oldLabels[nodeEgressLabel]
@@ -1067,23 +1077,6 @@ func (oc *Controller) getServiceLBInfo(lb, vip string) (string, bool) {
 	return conf.rejectACL, len(conf.endpoints) > 0
 }
 
-// getAllACLsForServiceLB retrieves all of the ACLs for a given load balancer
-func (oc *Controller) getAllACLsForServiceLB(lb string) []string {
-	oc.serviceLBLock.Lock()
-	defer oc.serviceLBLock.Unlock()
-	confMap, ok := oc.serviceLBMap[lb]
-	if !ok {
-		return nil
-	}
-	var acls []string
-	for _, v := range confMap {
-		if len(v.rejectACL) > 0 {
-			acls = append(acls, v.rejectACL)
-		}
-	}
-	return acls
-}
-
 // removeServiceLB removes the entire LB entry for a VIP
 func (oc *Controller) removeServiceLB(lb, vip string) {
 	oc.serviceLBLock.Lock()
